#!/usr/bin/env python3
# coding: utf-8

# can monitor for mks smart home
# base on https://python-ca.readthdocs.io/en/stable/configuration.html
# can configuration is in can.config
# (c) 2020 by Rainer Müller-Knoche
# last modified 25.08.2020

from __future__ import print_function
import sys

import can
import time

from can_MK import *

# **********************************************************************
# make hex output beautifull
# **********************************************************************
def padded_hex(i, l):
	given_int = i
	given_len = l

	hex_result = hex(given_int)[2:] # remove '0x' from beginning of str
	num_hex_chars = len(hex_result)
	extra_zeros = '0' * (given_len - num_hex_chars) # may not get used..

	return ('0x' + hex_result if num_hex_chars == given_len else
		'?' * given_len if num_hex_chars > given_len else
		'0x' + extra_zeros + hex_result if num_hex_chars < given_len else
			None)
# ======================================================================

# **********************************************************************
# decode message type
# **********************************************************************
def decode_t_type(t_type):
	s = "UNKNOWN"
	if t_type == D_BC:
		s = "Broadcast"
	elif t_type == D_GROUP:
		s = "Group"
	elif t_type == D_NODE:
		s = "Node"
	return s
# ======================================================================

# **********************************************************************
# decode Action
# **********************************************************************
def decode_action_1(data):
	a = "unknown " + hex(data)
	if data == T_OFF:
		a = "Switch Off"
	elif data == T_ON:
		a = "Switch_On"
	elif data == T_TOGGLE:
		a = "Toggle"
	elif data == T_TIMED:
		a = "Timed "
	return a
# ======================================================================

# **********************************************************************
# decode port
# **********************************************************************
def decode_action(data):
	a = "Target Port: "
	a = a + str(data)
	return a
# ======================================================================

# **********************************************************************
# ??
# **********************************************************************
def decode_u_action(data):
	s = "to be implemented"
	return s
# ======================================================================

# **********************************************************************
# decode conf subtypes
# **********************************************************************
def decode_conf_subtypes(data):
	if data == CONFRESET:
		s = "Reset"
	elif data == CONFREQUEST:
		s = "Config Request"
	elif data == CONFNEWID:
		s = "New ID"
	elif data == CONFNEWGR:
		s = "New Group"
	elif data == CONFNEW:
		s = "New Config"
	return s
# ======================================================================

# **********************************************************************
# decode information
# **********************************************************************
def decode_information(data):
	if data == S_TEMP:
		s = "Temperatur"
	elif data == S_PRESSURE:
		s = "Luftdruck"
	elif data == S_HUMIDITY:
		s = "Luftfeuchtigkeit"
	elif data == C_TIME:
		s = "Lokale Zeit"
	elif data == C_G_TIME:
		s = "Greenwich Zeit"
	elif data == C_SUNRAISE:
		s = "Sonnenaufgang"
	elif data == C_SUNSET:
		s = "Sonnenuntergang"
	return s
# ======================================================================

# **********************************************************************
# decode feedback data
# **********************************************************************
def decode_feedback(data):
	s = "port"
	return s
# ======================================================================

# **********************************************************************
# analyze message
# **********************************************************************
def decode_m_type(m_type, data,len):
	t = "t unknown"
	t1 = "t1 unknown"
	if m_type == REQUEST:
		t = "Request"
		t1 = ""
	elif m_type == ACTION:
		t = "Action"
		t1 = decode_action(data[M_DATA])			# target port
		t2 = decode_action_1(data[M_DATA+1])		# action
		t1 = t1 + " " + t2
		if len > 0 and len < 5:
			if data[M_DATA+2] == ISFEEDBACK:
				t1 = t1 + " Feedback "
			elif data[M_DATA+2] == GRFEEDBACK:
				t1 = t1 + " GrFeedBack "
			else:
				t1 = t1 + " noFeedback  "
			t1 = t1 + "Key " + str(data[M_DATA+3])
		if len > 0 and len == 5:
			t1 = t1 + str(data[M_DATA+3])
			if data[M_DATA] == T_TIMED:
				t1 = t1 + " ON for " + str(data[M_DATA+4]) + " Seconds"
	elif m_type == U_ACTION:# subtypes
		t = "U_Action"
		t1 = decode_u_action(data[M_DATA])
	elif m_type == L_INFO:
		t = "L_Info"
		t1 = ""
	elif m_type == ISFEEDBACK:
		t = "isFeedBack"
		t1 = decode_feedback(data[M_DATA]) 
	elif m_type == GRFEEDBACK:
		t = "GrFeedBack"
	elif m_type == ACK:
		t = "Acknoledge"
		t1 = ""
	elif m_type == DATA:
		t = "Data"
		t1 = ""
	elif m_type == SETTIME:
		t = "Set Time"
		t1 = ""
	elif m_type == CONF:# subtypes
		t = "Configuration Request"
		t1 = decode_conf_subtypes(data[M_DATA])
	elif m_type == INFO:# subtypes
		t = "Info"
		t1 = decode_information(data[M_DATA])
	t = t + " "
	t = t + t1
	return t 
# ======================================================================

# **********************************************************************
# decode a can message 11bit identifier
# **********************************************************************
def decode_short (canid, len, data):
	print("CAN ID:",padded_hex(canid,3))
	for i in range(0,len):
		print("Data+",i,padded_hex(data[i],2))

# ======================================================================

# **********************************************************************
# decode a can message
# **********************************************************************
def decode(canid, len, data):
	source = canid
	source = source >> 21
	s_ID = source
	source = source << 21
	target_type = canid
	target_type = target_type - source 
	target_type = target_type >> 19
	t_type = target_type
	target_type = target_type << 19
	message_type = canid
	message_type = message_type - source
	message_type = message_type - target_type
	message_type = message_type >> 15
	m_type = message_type
	message_type = message_type << 15
	target = canid
	target = target - source
	target = target - target_type
	target = target - message_type
	target = target >> 7
	t_ID = target
	print("Message Type:",decode_t_type(t_type))
#	if  t_type != D_BC:
	print("My Node ID:",padded_hex(PI_ID,2))
	print("Target Type:",
		padded_hex(t_type,1),
		decode_t_type(t_type),"from Node ID:",
		padded_hex(s_ID,2),"to Node ID:",
		padded_hex(t_ID,2))
	print("Message Type:",
		decode_m_type(m_type,data,len))
	for i in range(0,len):
		print("Data+",i,padded_hex(data[i],2))
#	if t_type == D_BC:
		
# ======================================================================

# **********************************************************************
# wait for a CAN message
# **********************************************************************
def wait_packet(bus):   				# wait for a packet
	print("waiting ...")
	while True: 						# received
		msg = bus.recv()				#
		print("Timestamp:",msg.timestamp)
		#print(msg.data)
		if msg.is_extended_id==False:
			#print("short")
			decode_short(msg.arbitration_id,
				msg.dlc,
				msg.data)
		else:
			decode(msg.arbitration_id,
				msg.dlc,
				msg.data)
# ======================================================================

# **********************************************************************
# main entry
# **********************************************************************
if __name__ == '__main__':
	global bc
	print("CAN Monitor")
	bus = can.interface.Bus(bustype='socketcan', channel='can0', bitrate=250000)   # bus
	if str(sys.argv) == "y":
		bc = 1
	while True:
		wait_packet(bus)					#
# ======================================================================
	
