#!/usr/bin/env python3
# coding: utf-8

# get config from node, store it to file
# base on https://python-ca.readthdocs.io/en/stable/configuration.html
# can configuration is in can.config
# (c) 2020 by Rainer Müller-Knoche
# last modified 10.03.2020

from __future__ import print_function

import can
import time

from can_MK import *

# send one message
def send_one(bus,msg,target_ID):
    #msg = can.Message(arbitration_id=0xc0ffee,
    #                  data=[0x7F, 0x02, 0x01, 0x01, 0x00, 0x02],
    #                  is_extended_id=True)
    try:
        bus.send(msg)
        #print("Message sent on {}".format(bus.channel_info))
        print("Message sent to node: ",target_ID)
    except can.CanError:
        print("Message NOT sent")
#=================

# wait for an acknoledge of a send message
# return false if timeout of 3 seconds
# we check if is from the node we sent the data
# we ignore all other messages
def wait_ack(bus,target_ID):            # wait acknolage
    timeout = time.time() + 3           # we will wait max 3 seconds
    while True:                         # 
        msg = bus.recv()                # get the receive message
        decode(msg.arbitration_id)      # decode ID to global values
        if M_type == ACK and S_ID == target_id: # check if ACK messag and from target
                return True             # return true on ACK
        if time.time() > timeout:
            return False                # return false on tmeout
#=================

# decode can ID
# we return a global set of decoded information
def decode_id(canid):
    global S_ID
    global T_type
    global M_type
    global T_ID
    
    source = canid
    source = source >> 21
    S_ID = source
    source = source << 21
    target_type = canid
    target_type = target_type - source 
    target_type = target_type >> 19
    T_type = target_type
    target_type = target_type << 19
    message_type = canid
    message_type = message_type - source
    message_type = message_type - target_type
    message_type = message_type >> 15
    M_type = message_type
    message_type = message_type << 15
    target = canid
    target = target - source
    target = target - target_type
    target = target - message_type
    target = target >> 7
    T_ID = target
# =============

# encode the canID from parameters
# source (sender) ID is fixed to our ID (PI_ID)
# we return the encoded ID
def encode_id(t_type, m_type, t_ID):
    tmp_id = PI_ID
    tmp_id = tmp_id << 21
    can_id = tmp_id
    tmp_id = t_type
    tmp_id = tmp_id << 19
    can_id = can_id + tmp_id
    tmp_id = m_type
    tmp_id = tmp_id << 15
    can_id = can_id + tmp_id
    tmp_id = t_ID
    tmp_id = tmp_id << 7
    can_id = can_id + tmp_id
    return can_id
# ==============

# we requested a config
# so we have to get the amount of expected data
# we timeout if no answer in 3 seconds
# we check if it's from the node we requested
# we ignore every other message
# NOTE length is only one byte, so max length is 255
def wait_linfo(bus,target_ID):
    timeout = time.time() + 3                       # set timeout
    while True:
        msg = bus.recv()                            # get a message
        decode_id(msg.arbitration_id)               # decode ID
        if M_type == L_INFO and S_ID == target_ID:  # check
            return msg.data[M_DATA]                 # data amount 
        if time.time() > timeout:
            return False
# ================

# send an Acknoledge to source
# source may send next messge
def send_ack(bus, target_ID):
    msg = can.Message(arbitration_id=
            encode_id(D_NODE,ACK,target_ID),
            data=[],
            is_extended_id=True)
    send_one(bus,msg,target_ID)
# ================

# we request a complete config an store is localy 
def get_config(target_ID,f):
    bus = can.interface.Bus(bustype='socketcan', channel='can0', bitrate=250000)                           # init CAN bus
    msg = can.Message(arbitration_id=
            encode_id(D_NODE,CONF,target_ID),           # encode header 
            data=[CONFREQUEST],                         # send config request
            is_extended_id=True)                        # 
    send_one(bus,msg,target_ID)                         # send message
    to_receive =  wait_linfo(bus,target_ID)             # wait Ack
    print("expect ",to_receive," bytes")
    if not to_receive:
        print("NO ANSWER")                              # TIMEOUT
        return False
    else:
        try:
            tmpfile = open(f,"w")                       # ok we can open a file
            send_ack(bus, target_ID)                    # send ack = request next msg
            timeout = time.time() + 3                   # set timeout 3 seconds
            while True:
                if time.time() > timeout:
                    print("TIME OUT")                   # TIMEOUT
                    tmpfile.close()
                    return False
                msg = bus.recv()                        # get message
                decode_id(msg.arbitration_id)           # decode to global
                if M_type == DATA and S_ID == target_ID:
                    if to_receive >= 8:                 # we will get 8 byte max
                        to_receive = to_receive - 8     # dec data amount
                        print("to_receive ",to_receive)
                        for i in range(0,8):
                            print("i: ",i," ",padded_hex(msg.data[i],2))
                            to_write = padded_hex(msg.data[i],2)
                            to_write += ","
                            tmpfile.write(to_write)     # write 1 values)
                        print("Block written")
                        send_ack(bus, target_ID)
                    else:   # we have less than 8 bytes, the rest
                        # NOTE should work if rest is 0 bytes
                        print("less to_receive ",to_receive)
                        for i in range(0,to_receive):
                            print("i: ",i," ",padded_hex(msg.data[i],2))
                            to_write = padded_hex(msg.data[i],2)
                            to_write += ","
                            tmpfile.write(to_write)
                        send_ack(bus, target_ID)
                        tmpfile.close()
                        return True     # do not wait for more data
        finally:
            tmpfile.close()
            print("finally END")

# ============

def padded_hex(i, l):
    given_int = i
    given_len = l

    hex_result = hex(given_int)[2:] # remove '0x' from beginning of str
    num_hex_chars = len(hex_result)
    extra_zeros = '0' * (given_len - num_hex_chars) # may not get used..

    return ('0x' + hex_result if num_hex_chars == given_len else
            '?' * given_len if num_hex_chars > given_len else
            '0x' + extra_zeros + hex_result if num_hex_chars < given_len else
            None)
# ============

def get_id():
    id = input("Node ID? ")
    print(id)
    y = input("OK (yes/no)? ")
    if y == "yes":
        return id 
    else:
        return False

if __name__ == '__main__':
    id = get_id()
    if not id:
        print("Sorry ")
        exit()
    id_hex = int(id,16)
    id = id.upper()
    f = "node_"
    f += id
    f += ".recv"
    print(f)
    get_config(id_hex,f)
