#!/usr/bin/env python3
# coding: utf-8

# reset node
# based on https://python-ca.readthdocs.io/en/stable/configuration.html
# can configuration is in can.config
# (c) 2020 by Rainer Müller-Knoche
# last modified 6. september 2020

from __future__ import print_function

import sys
import can
import time

from can_MK import *

bus = can.interface.Bus(bustype='socketcan', channel='can0', bitrate=250000)   # init CAN bus

# send one message
def send_one(bus,msg,target_ID):
	# msg = can.Message(arbitration_id=0xc0ffee,
	#		  data=[0x7F, 0x02, 0x01, 0x01, 0x00, 0x02],
	#		  is_extended_id=True)
	try:
		bus.send(msg)
		#print("Message sent on {}".format(bus.channel_info))
		print("Message sent to node: ",target_ID)
	except can.CanError:
		print("Message NOT sent")
# =================

# wait for an acknoledge of a send message
# return false if timeout of 3 seconds
# we check if is from the node we sent the data
# we ignore all other messages
def wait_ack(bus,target_ID):			# wait acknolage
	timeout = time.time() + 3			# we will wait max 3 seconds
	while True:							#
		msg = bus.recv()				# get the receive message
		decode(msg.arbitration_id)	 	# decode ID to global values
		if M_type == ACK and S_ID == target_id: # check if ACK messag and from target
				return True				# return true on ACK
		if time.time() > timeout:
			return False				# return false on tmeout
#=================


# encode the canID from parameters
# source (sender) ID is fixed to our ID (PI_ID)
# we return the encoded ID
def encode_id(t_type, m_type, t_ID):
	tmp_id = PI_ID
	tmp_id = tmp_id << 21
	can_id = tmp_id
	tmp_id = t_type
	tmp_id = tmp_id << 19
	can_id = can_id + tmp_id
	tmp_id = m_type
	tmp_id = tmp_id << 15
	can_id = can_id + tmp_id
	tmp_id = t_ID
	tmp_id = tmp_id << 7
	can_id = can_id + tmp_id
	return can_id
# ==============


# send a reset request to node
def send_reset(bus, target_ID,res_time):
	msg = can.Message(arbitration_id=				#
		encode_id(D_NODE, CONF, target_ID),
		data=[CONFRESET,res_time],					# send config request
		is_extended_id=True)						#
	send_one(bus,msg,target_ID)						# send message
# ============



if __name__ == '__main__':
	a = sys.argv[1]
	a = int(a)
	b = sys.argv[2]
	b = int(b)
	send_reset(bus, a, b)
