#!/usr/bin/env python3
# coding: utf-8

# setid send new id to node
# based on https://python-ca.readthdocs.io/en/stable/configuration.html
# can configuration is in can.config
# (c) 2020 by Rainer Müller-Knoche
# last modified 10.03.2020

from __future__ import print_function

import can
import time

from can_MK import *

# send one message
def send_one(bus,msg,target_ID):
    #msg = can.Message(arbitration_id=0xc0ffee,
    #                  data=[0x7F, 0x02, 0x01, 0x01, 0x00, 0x02],
    #                  is_extended_id=True)
    try:
        bus.send(msg)
        #print("Message sent on {}".format(bus.channel_info))
        print("Message sent to node: ",target_ID)
    except can.CanError:
        print("Message NOT sent")
# =================

# wait for an acknoledge of a send message
# return false if timeout of 3 seconds
# we check if is from the node we sent the data
# we ignore all other messages
def wait_ack(bus,target_ID):            # wait acknolage
    timeout = time.time() + 3           # we will wait max 3 seconds
    while True:                         #
        msg = bus.recv()                # get the receive message
        decode(msg.arbitration_id)      # decode ID to global values
        if M_type == ACK and S_ID == target_id: # check if ACK messag and from target
                return True             # return true on ACK
        if time.time() > timeout:
            return False                # return false on tmeout
#=================

# decode can ID
# we return a global set of decoded information
def decode_id(canid):
    global S_ID
    global T_type
    global M_type
    global T_ID

    source = canid
    source = source >> 21
    S_ID = source
    source = source << 21
    target_type = canid
    target_type = target_type - source
    target_type = target_type >> 19
    T_type = target_type
    target_type = target_type << 19
    message_type = canid
    message_type = message_type - source
    message_type = message_type - target_type
    message_type = message_type >> 15
    M_type = message_type
    message_type = message_type << 15
    target = canid
    target = target - source
    target = target - target_type
    target = target - message_type
    target = target >> 7
    T_ID = target
# =============

# encode the canID from parameters
# source (sender) ID is fixed to our ID (PI_ID)
# we return the encoded ID
def encode_id(t_type, m_type, t_ID):
    tmp_id = PI_ID
    tmp_id = tmp_id << 21
    can_id = tmp_id
    tmp_id = t_type
    tmp_id = tmp_id << 19
    can_id = can_id + tmp_id
    tmp_id = m_type
    tmp_id = tmp_id << 15
    can_id = can_id + tmp_id
    tmp_id = t_ID
    tmp_id = tmp_id << 7
    can_id = can_id + tmp_id
    return can_id
# ==============

# send a new id to node
def send_id(bus, target_ID, new_ID):
    msg = can.Message(arbitration_id=
        encode_id(D_NODE, CONF, target_ID),
        data=[CONFNEWID, new_ID],                       # new ID
        is_extended_id=True)
    send_one(bus,msg,target_ID)                         # send message
# ============

# send a reset request to node
def send_reset(bus, target_ID):
    msg = can.Message(arbitration_id=                   #
        encode_id(D_NODE, CONF, target_ID),
        data=[CONFRESET],                               # send config request
        is_extended_id=True)                            #
    send_one(bus,msg,target_ID)                         # send message
# ============

def get_id():
    id = input("Node ID? ")
    print(id)
    y = input("OK (yes/no)? ")
    if y == "yes":
        return id 
    else:
        return False
# =================

if __name__ == '__main__':
    id = get_id()
    if not id:
        print("Sorry ")
        exit()
    id_hex = int(id,16)
    print("New ID now")
    new_id = get_id()
    if not new_id:
        print("Sorry ")
    new_id_hex = int(new_id,16)
    bus = can.interface.Bus(bustype='socketcan', channel='can0', bitrate=250000)                                       # init CAN bus
    send_id(bus,id_hex, new_id_hex)
    send_reset(bus, id_hex)
