#!/usr/bin/env python3
# coding: utf-8

# write config, we write a complete new config to node
# base on https://python-ca.readthdocs.io/en/stable/configuration.html
# can configuration is in can.config
# (c) 2020 by Rainer Müller-Knoche
# last modified 10.03.2020


from __future__ import print_function

import can
import time

from can_MK import *

bus = can.interface.Bus(bustype='socketcan', channel='can0', bitrate=250000)	# init CAN bus

# **********************************************************************
# make hex output beautifull
# **********************************************************************
def padded_hex(i, l):
	given_int = i
	given_len = l

	hex_result = hex(given_int)[2:] # remove '0x' from beginning of str
	num_hex_chars = len(hex_result)
	extra_zeros = '0' * (given_len - num_hex_chars) # may not get used..

	return ('0x' + hex_result if num_hex_chars == given_len else
		'?' * given_len if num_hex_chars > given_len else
		'0x' + extra_zeros + hex_result if num_hex_chars < given_len else
			None)
# ======================================================================

# **********************************************************************
# send one message
#***********************************************************************
def send_one(bus,msg,target_ID):
	#msg = can.Message(arbitration_id=0xc0ffee,
	#				  data=[0x7F, 0x02, 0x01, 0x01, 0x00, 0x02],
	#				  is_extended_id=True)
	try:
		bus.send(msg)
		#print("Message sent on {}".format(bus.channel_info))
		print("Message sent to node: ",target_ID)
	except can.CanError:
		print("Message NOT sent")

# ======================================================================

# wait for an acknoledge of a send message
# return false if timeout of 3 seconds
# we check if is from the node we sent the data
# we ignore all other messages
def wait_ack(bus,target_ID):			# wait acknolage
	timeout = time.time() + 3		   # we will wait max 3 seconds
	while True:						 #
		msg = bus.recv()				# get the receive message
		decode_id(msg.arbitration_id)	  # decode ID to global values
		if M_type == ACK and S_ID == target_ID: # check if ACK messag and from target
				return True			 # return true on ACK
		if time.time() > timeout:
			return False				# return false on tmeout
# =================

# send an Acknoledge to source
# source may send next messge
def send_ack(bus, target_ID):
	msg = can.Message(arbitration_id=
			encode_id(D_NODE,ACK,target_ID),
			data=[],
			is_extended_id=True)
	send_one(bus,msg,target_ID)
# ================

# encode the canID from parameters
# source (sender) ID is fixed to our ID (PI_ID)
# we return the encoded ID
def encode_id(t_type, m_type, t_ID):
	tmp_id = PI_ID
	tmp_id = tmp_id << 21
	can_id = tmp_id
	tmp_id = t_type
	tmp_id = tmp_id << 19
	can_id = can_id + tmp_id
	tmp_id = m_type
	tmp_id = tmp_id << 15
	can_id = can_id + tmp_id
	tmp_id = t_ID
	tmp_id = tmp_id << 7
	can_id = can_id + tmp_id
	return can_id
# ==============

# decode can ID
# we return a global set of decoded information
def decode_id(canid):
	global S_ID
	global T_type
	global M_type
	global T_ID

	source = canid
	source = source >> 21
	S_ID = source
	source = source << 21
	target_type = canid
	target_type = target_type - source
	target_type = target_type >> 19
	T_type = target_type
	target_type = target_type << 19
	message_type = canid
	message_type = message_type - source
	message_type = message_type - target_type
	message_type = message_type >> 15
	M_type = message_type
	message_type = message_type << 15
	target = canid
	target = target - source
	target = target - target_type
	target = target - message_type
	target = target >> 7
	T_ID = target
# =============

def read_config(node,target_ID):
	i = 0													# declare counter 
	tmp = open("tmpconf","w")								# open a empty temporary file
	with open(node, "r") as searchfile:						# open source file node config
		for line in searchfile:								# for everylin in source file
			config_values  = line							# get one line
			config_values = "".join(config_values.split())	# remove whitespace 
			if not config_values.startswith("/"):			# skip comment lines
				if not config_values.startswith("}"):		# skip final bracket
					rest, sep, tail = config_values.partition("/")  # remove trailing coment
					rest = rest.replace("\n","")			# remove new line
					rest = rest.rstrip()					# remove remaining white space
					for ch in rest:							# every char
						tmp.write(ch)						# write pure config information
						if ch == "x":						# to tempfile
							i += 1						 	# count them
	tmp.write(",")
	print("total bytes: ",i)								# for debug print count
	tmp.close()												# close temp file

	tmp = open("tmpconf","r")								# reopen temp file
#	bus = can.interface.Bus(bustype='socketcan', channel='can0', bitrate=250000)	# init CAN bus
	msg = can.Message(arbitration_id=
			encode_id(D_NODE, L_INFO, target_ID),			# send start message L_Info
			data=[CONF, CONFNEW, i],						# 
			is_extended_id=True)							# or extracted from source file
	send_one(bus,msg,target_ID)								# send message
	if wait_ack(bus,target_ID):								# wait Ack
		count = 0											# init send counter
		print("i =",i)										# for debug
		while count < i:									# while sent bytes less to sent
			value = []										# init tempory send buffer
			dif = i - count									# check if still 4 bytes to send
			if dif >= 8:									# if yes send 8 bytes
				for n in range(0,8):						# fill tempory buffer
					value.append(tmp.read(5))				# from file
				count = count + 8							# increment counterby sent bytes
				print("Sent bytes: ",count)
				mydata = []
				for j in range(0,8):
					x = value[j]
					x = x.replace(",","")
					x = int(x,16)
					mydata.append(x)
				print("sent Data: ",mydata)
				msg = can.Message(arbitration_id=
						encode_id(D_NODE, DATA, target_ID),
						data=mydata,
						is_extended_id=True)
				send_one(bus, msg, target_ID)
				if not wait_ack(bus,target_ID):
					print("TIME OUT")
					break
			else:
				for n in range(0,dif):
					value.append(tmp.read(5))
				count = count + dif
				print("Sent bytes:",count)
				mydata = []
				for j in range(0,dif):
					x = value[j]
					x = x.replace(",","")
					x = int(x,16)
					mydata.append(x)
				print("Sent Data: ",mydata)
				msg = can.Message(arbitration_id=
						encode_id(D_NODE, DATA, target_ID),
						data=mydata,
						is_extended_id=True)
				send_one(bus, msg, target_ID)
				if not wait_ack(bus,target_ID):
					print("END without ACK")
	else:
		print("NO ANSWER")
	tmp.close()
# ============

# send a reset request to node
def send_reset(bus, target_ID):
	print("All done RESET now")
	msg = can.Message(arbitration_id=					#
		encode_id(D_NODE, CONF, target_ID),
		data=[CONFRESET],								# send config request
		is_extended_id=True)							#
	send_one(bus,msg,target_ID)							# send message
# ============


def get_id():
	canid = input("Node ID? ")
	print(canid)
	y = input("OK (yes/no)? ")
	if y == "yes":
		return canid
	else:
		return False

if __name__ == '__main__':
	canid = get_id()
	if not canid:
		print("Sorry ")
		exit()
	id_hex = int(canid,16)
	canid = canid.upper()
	f = "node_"
	f += canid
	read_config(f,id_hex)
	send_reset(bus,id_hex)
# ===== END ====
